@@ -275,7 +275,7 @@ local function sortTable(tbl)
 end
 
 --- 创建一个有序表
----@param tbl table {optional = 'self'}
+---@param tbl? table
 ---@return table
 function m.container(tbl)
     return sortTable(tbl)
@@ -290,6 +290,9 @@ function m.loadFile(path, keepBom)
     end
     local text = f:read 'a'
     f:close()
+    if not text then
+        return nil
+    end
     if not keepBom then
         if text:sub(1, 3) == '\xEF\xBB\xBF' then
             return text:sub(4)
@@ -524,6 +527,14 @@ function m.revertTable(t)
     return t
 end
 
+function m.revertMap(t)
+    local nt = {}
+    for k, v in pairs(t) do
+        nt[v] = k
+    end
+    return nt
+end
+
 function m.randomSortTable(t, max)
     local len = #t
     if len <= 1 then
@@ -570,7 +581,7 @@ end
 ---遍历文本的每一行
 ---@param text string
 ---@param keepNL? boolean # 保留换行符
----@return fun(text:string):string, integer
+---@return fun():string, integer
 function m.eachLine(text, keepNL)
     local offset = 1
     local lineCount = 0
@@ -719,7 +730,7 @@ function switchMT:has(name)
 end
 
 ---@param name string
----@return ...
+---@return any ...
 function switchMT:__call(name, ...)
     local callback = self.map[name] or self._default
     if not callback then
@@ -791,6 +802,8 @@ function m.multiTable(count, default)
     return current
 end
 
+---@param t table
+---@param sorter boolean|function
 function m.getTableKeys(t, sorter)
     local keys = {}
     for k in pairs(t) do
@@ -798,7 +811,7 @@ function m.getTableKeys(t, sorter)
     end
     if sorter == true then
         tableSort(keys)
-    elseif sorter then
+    elseif type(sorter) == 'function' then
         tableSort(keys, sorter)
     end
     return keys
